#include "memorycard.h"
#include "memorygameboard.h"

MemoryCard::MemoryCard(QDeclarativeItem *parent) :
    QDeclarativeItem(parent),
    _isFace(false)
{
    setFlag(QGraphicsItem::ItemHasNoContents, false);
    setAcceptedMouseButtons(Qt::LeftButton);
}

MemoryCard::MemoryCard(QDeclarativeItem *parent, unsigned id) :
    QDeclarativeItem(parent),
    _isFace(false),
    _id(id)
{
    setFlag(QGraphicsItem::ItemHasNoContents, false);
    setAcceptedMouseButtons(Qt::LeftButton);
}

void MemoryCard::flip(const char *slotName)
{
    QParallelAnimationGroup *group = new QParallelAnimationGroup(this);

    QPropertyAnimation *posAnimation = new QPropertyAnimation(this, "pos", this);
    posAnimation->setStartValue(pos());
    posAnimation->setKeyValueAt(0.5, QPointF(x() + (width() * 0.05), y() + (height() * 0.05)));
    posAnimation->setEndValue(pos());
    posAnimation->setDuration(200);
    posAnimation->setEasingCurve(QEasingCurve::OutInExpo);

    QPropertyAnimation *scaleAnimation = new QPropertyAnimation(this, "scale", this);
    scaleAnimation->setStartValue(1);
    scaleAnimation->setKeyValueAt(0.5, 0.7);
    scaleAnimation->setEndValue(1);
    scaleAnimation->setDuration(200);
    scaleAnimation->setEasingCurve(QEasingCurve::OutInExpo);

    group->addAnimation(posAnimation);
    group->addAnimation(scaleAnimation);
    QTimer::singleShot(100, this, slotName);
    group->start(QAbstractAnimation::DeleteWhenStopped);
}

void MemoryCard::flipToBack()
{
    flip(SLOT(setToBack()));
}

void MemoryCard::flipToFace()
{
    flip(SLOT(setToFace()));
}

void MemoryCard::setToBack()
{
    _isFace = false;
}

void MemoryCard::setToFace()
{
    _isFace = true;
}

bool MemoryCard::isFace() const
{
    return _isFace;
}

void MemoryCard::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    MemoryGameBoard *board = (MemoryGameBoard*)parent();

    if (!_isFace && board->canReveal())
    {
        flipToFace();

        if (board->lastRevealed())
        {
            MemoryCard *other = board->lastRevealed();
            board->setLastRevealed(0);

            if (this->_id == other->_id)
            {
                QTimer::singleShot(500, this, SLOT(flyOut()));
                QTimer::singleShot(500, other, SLOT(flyOut()));
                QTimer::singleShot(900, board, SLOT(enableReveal()));
            }
            else
            {
                QTimer::singleShot(1000, this, SLOT(flipToBack()));
                QTimer::singleShot(1000, other, SLOT(flipToBack()));
                QTimer::singleShot(1000, board, SLOT(enableReveal()));
            }
        }
        else
        {
            board->setLastRevealed(this);
        }
    }

    event->accept();
}

void MemoryCard::flyOut()
{
    QPropertyAnimation *animation = new QPropertyAnimation(this, "pos", this);
    animation->setEndValue(QPoint(- 10 * width(), - 10 * height()));
    animation->setDuration(400);
    animation->setEasingCurve(QEasingCurve::InElastic);
    connect(animation, SIGNAL(finished()), this, SIGNAL(matched()));
    connect(animation, SIGNAL(finished()), this, SLOT(deleteLater()));
    animation->start(QAbstractAnimation::DeleteWhenStopped);
}

void MemoryCard::saveData(QDataStream &stream) const
{
    stream << _id << _isFace << pos();
}

void MemoryCard::loadData(QDataStream &stream)
{
    QPointF pos;
    stream >> _id >> _isFace >> pos;
    setPos(pos);
}

void MemoryCard::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);

    QColor bg;
    QColor fg;
    QFont font;
    font.setPixelSize(height() - 15);

    if (!_isFace)
    {
        bg = QColor(159, 206, 0, 255);
        fg = QColor(255, 255, 255, 255);
    }
    else
    {
        bg = QColor(230, 230, 230, 255);
        fg = QColor(20, 20, 20, 255);
    }
    painter->setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
    painter->setFont(font);
    painter->setPen(bg);
    painter->setBrush(bg);
    painter->drawRoundedRect(0, 0, width(), height(), 10, 10, Qt::RelativeSize);
    painter->setPen(fg);

    if (_isFace)
    {
        // TODO: later allow drawing of pixmaps here
        painter->drawText(0, 0, width(), height(), Qt::AlignCenter, QString(_id));
    }
    else
    {
        painter->drawText(0, 0, width(), height(), Qt::AlignCenter, "?");
    }
}
