#include "memorygameboard.h"

MemoryGameBoard::MemoryGameBoard(QDeclarativeItem *parent) :
    QDeclarativeItem(parent),
    #if defined(Q_WS_MAEMO_5)
    _rows(3),
    _columns(7),
    _margin(10),
    #else
    _rows(3),
    _columns(9),
    _margin(10),
    #endif
    _elapsedSteps(0),
    _lastRevealed(0),
    _canReveal(true),
    _isGameStarted(false)
{
}

unsigned MemoryGameBoard::rows() const
{
    return _rows;
}

void MemoryGameBoard::setRows(unsigned value)
{
    if (!_isGameStarted)
    {
        _rows = value;
        emit rowsChanged();
    }
}

unsigned MemoryGameBoard::columns() const
{
    return _columns;
}

void MemoryGameBoard::setColumns(unsigned value)
{
    if (!_isGameStarted)
    {
        _columns = value;
        emit columnsChanged();
    }
}

bool MemoryGameBoard::isGameStarted() const
{
    return _isGameStarted;
}

unsigned MemoryGameBoard::elapsedSteps() const
{
    return _elapsedSteps;
}

MemoryCard *MemoryGameBoard::lastRevealed() const
{
    return _lastRevealed;
}

void MemoryGameBoard::setLastRevealed(MemoryCard *card)
{
    if (_canReveal)
    {
        if (card == 0)
        {
            _canReveal = false;
            _elapsedSteps++;
            emit elapsedStepsChanged();
        }
        _lastRevealed = card;
    }
}

bool MemoryGameBoard::canReveal() const
{
    return _canReveal;
}

void MemoryGameBoard::enableReveal()
{
    _canReveal = true;
}

void MemoryGameBoard::startGame()
{
    if (_isGameStarted)
        return;

    qDeleteAll(items);
    items.clear();
    _isGameStarted = true;
    emit isGameStartedChanged();

    unsigned r = width() > height() ? _rows : _columns;
    unsigned c = width() < height() ? _rows : _columns;

    qreal w = (qreal) (width() - 2 * _margin) / (qreal)c - _margin;
    qreal h = (qreal) (height() - 2 * _margin) / (qreal)r - _margin;

    QFont font;
    font.setPixelSize(h - 30);

    QParallelAnimationGroup *group = new QParallelAnimationGroup(this);
    QList<char> chars = generateChars(r * c);
    bool skipLast = (r * c) % 2;

    qsrand(QTime::currentTime().msec());

    for (unsigned i = 0; i < r; i++)
    {
        for (unsigned j = 0; j < c; j++)
        {
            if (skipLast && j == c - 1 && i == r - 1)
            {
                continue;
            }

            char ch = chars[qrand() % chars.count()];
            chars.removeOne(ch);

            MemoryCard *card = new MemoryCard(this, ch);
            card->setWidth(w);
            card->setHeight(h);

            QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(card);
            effect->setColor(QColor(0, 0, 0, 200));
            effect->setOffset(0);
            effect->setBlurRadius(_margin * 1.2);
            card->setGraphicsEffect(effect);

            items.append(card);
            connect(card, SIGNAL(matched()), this, SLOT(cardMatched()));
            card->show();

            QPropertyAnimation *animation = new QPropertyAnimation(card, "pos", this);
            animation->setStartValue(QPointF(j * w + (j + 1) * _margin, -h));
            animation->setEndValue(QPointF(j * w + (j + 1) * _margin, i * h + (i + 1) * _margin));
            animation->setDuration(1000);
            animation->setEasingCurve(QEasingCurve::OutBounce);
            group->addAnimation(animation);
        }
    }

    enableReveal();
    _elapsedSteps = 0;
    emit elapsedStepsChanged();
    group->start(QAbstractAnimation::DeleteWhenStopped);
}

void MemoryGameBoard::surrenderGame()
{
    foreach (MemoryCard *card, items)
        card->flipToFace();
    _canReveal = false;
    _lastRevealed = 0;

    _isGameStarted = false;
    emit isGameStartedChanged();
}

void MemoryGameBoard::cardMatched()
{
    items.removeOne((MemoryCard*)sender());
    if (items.count() == 0)
    {
        _isGameStarted = false;
        emit isGameStartedChanged();
        emit gameWon();
    }
}

QList<char> MemoryGameBoard::generateChars(unsigned n)
{
    QList<char> chars;
    if (n % 2)
        n--;
    unsigned i = 0;
    for (char c = '0'; i < n && c <= '9'; c+=1, i+=2)
    {
        chars.push_back(c);
        chars.push_back(c);
    }
    for (char c = 'A'; i < n && c <= 'Z'; c+=1, i+=2)
    {
        chars.push_back(c);
        chars.push_back(c);
    }
    return chars;
}

void MemoryGameBoard::saveData(QDataStream &stream) const
{
    stream << _rows << _columns << _margin << _elapsedSteps << _canReveal << items.count() << items.indexOf(_lastRevealed);
    foreach(MemoryCard *card, items)
        card->saveData(stream);
}

void MemoryGameBoard::loadData(QDataStream &stream)
{
    int cardCount, lastRevealedIndex;
    stream >> _rows >> _columns >> _margin >> _elapsedSteps >> _canReveal >> cardCount >> lastRevealedIndex;
    for (int i = 0; i < cardCount; i++)
    {
        MemoryCard *card = new MemoryCard(this);

        card->loadData(stream);
        QGraphicsDropShadowEffect *effect = new QGraphicsDropShadowEffect(card);
        effect->setColor(QColor(0, 0, 0, 200));
        effect->setOffset(0);
        effect->setBlurRadius(_margin * 1.2);
        card->setGraphicsEffect(effect);

        items.append(card);
        connect(card, SIGNAL(matched()), this, SLOT(cardMatched()));
        card->show();

        if (i == lastRevealedIndex)
            _lastRevealed = card;
    }
    _isGameStarted = true;
    emit isGameStartedChanged();
}

void MemoryGameBoard::changeOrientation()
{
    unsigned r = width() > height() ? _rows : _columns;
    unsigned c = width() < height() ? _rows : _columns;

    qreal w = (qreal) (width() - 2 * _margin) / (qreal)c - _margin;
    qreal h = (qreal) (height() - 2 * _margin) / (qreal)r - _margin;

    // TODO: the items should keep their position in the grid instead
    for (unsigned i = 0; i < r; i++)
    {
        for (unsigned j = 0; j < c; j++)
        {
            unsigned val = i * c + j;
            if (val >= items.count())
            {
                return;
            }

            MemoryCard *card = items[val];
            card->setWidth(w);
            card->setHeight(h);
            card->setPos(QPointF(j * w + (j + 1) * _margin, i * h + (i + 1) * _margin));
        }
    }
}
